/*--------------------------------------------------------------------------
	File-name	: parse.c
	Description	: Example of calling Korean dependency parsing module.
			  Hangul I/O code is KSC-5601-1987 or KSSM Hangul code.
	Written-by	: Kang, Seung-Shik at Hansung Univ.  1997. 11.
----------------------------------------------------------------------------*/
#include "../header/ham-par.h"
#include <ctype.h>	/* needed only for 'tolower()' */

extern int get_sent_or_line();	/* src/readsent.c */
extern void put_dep_relation();
extern void put_dep_tree();	/* src/dpar-out.c */

/*
	Set HAM options at the command-line argument.
*/
void usage_parse()
{
	puts("usage: parse [-options] [input.txt] [output.txt]\n");
	puts("    no options & I/O files --> default options applied\n");
	puts("\t-c: make noun+noun+...+noun nodes to 1 node\n");

	puts("\t-A: automatic spacing for input sentence");
	puts("\t-t: print-out parse tree: default");
	puts("\t-f: print-out parse-failed subtree, too");
	puts("\t-T: don't print-out parse tree\n");

	puts("\t-m: don't append morph. analysis result");
	puts("\t-i: append dic-info. to morph result");
	puts("\t-b: append mother & neighbor stem\n");

	puts("\t-d: print-out dependency parse tree, too");
	puts("\t-D: print-out dependency parse & relation\n");

	puts("\t-M: print-out morph. result before parsing");
	puts("\t-n: print all nodes by word-id & their govenor\n");

	puts("\t-s: don't echo-back input sentence itself");
	puts("\t-l: input mode: line-by-line --> sentence mark");
	puts("\t-r: select a random sentence at input file\n");
	puts("    Options may be combined like -m, -md, or -nl");
	puts("    WITH OPTION & NO I/O FILES SPECIFIED --> INTERACTIVE TESTING");
	GOODBYE;	/* goodbye message: 'header/ham-par.h' */
}

/*
	Set I/O types: following 3 types are possible.
		1. stdin & stdout
		2. file input & stdout
		3. file input & file output
*/
int set_iofile_ptr(argc, argv, fpin, fpout)
int argc; char *argv[];
FILE **fpin, **fpout;
{
	int i=1, nargs=argc;

	if (argc > 1 && argv[1][0] == '-') { i++; nargs--; }

	switch (nargs) {
	case 1:
		if (argc == 1) { usage_parse(); return 1; }
		break;
	case 2:
		*fpin = fopen(argv[i], "r");
		if (!*fpin) { printf("No such file <%s>\n", argv[i]); return 1; }
		break;
	case 3:
		*fpin = fopen(argv[i], "r");
		if (!*fpin) { printf("No such file <%s>\n", argv[i]); return 1; }
		if (*fpout = fopen(argv[i+1], "r")) {
			fprintf(stderr, "Overwrite output file <%s> ? ", argv[i+1]);
			if (tolower(getchar()) != 'y') return 1;
		}
		*fpout = fopen(argv[i+1], "w");
		break;
	default:
		usage_parse(); return 1;
	}

	return 0;
}

/*
	Recursive print-out head-daughter parsing output.
*/
void put_tree(fp, node, depth, mode)
FILE *fp;
KPAR_PWORD node;	/* current node */
int depth;	/* depth of current node */
HAM_PRUNMODE mode;	/* running mode of HAM */
{
	int i, j;
	int opt = mode->parse.put_morp;
	KPAR_PWORD p;

	if (node == NULL) return;

	for (j = 0; j < depth; j++) fprintf(fp, "    ");	/* indentation */
	fprintf(fp, "%c %s", node->rel, node->word);

	fprintf(fp, " %c:%s", node->bpos, node->base);
	if (node->pomi != NULL) fprintf(fp, " P:%s", node->pomi);
	if (node->eomi != NULL) fprintf(fp, " E:%s", node->eomi);
	if (node->josa != NULL) fprintf(fp, " J:%s", node->josa);

	fprintf(fp, "\n");
	/*append_more_info(fp, node, depth, mode);	/* morp. result? */

	if (node->nchild) {	/* daughters */
		p = node->last;
		for (i = node->nchild-1; i >= 0; i--) {
			put_tree(fp, p, depth+1, mode);
			p = p->left;
		}
	}
}

/*
	Print-out test of head-daughter structure.
*/
void parsing_out(fp, kpr, mode)
FILE *fp;
KPAR_PSENT kpr;	/* head-daughter parse tree */
HAM_PRUNMODE mode;	/* running mode of HAM */
{
	/* output test for dependency relation */
	if (mode->parse.put_depn == 1)
		put_dep_relation(fp, kpr->dpr);
	if (mode->parse.put_depn == 2)
		put_dep_tree(fp, kpr->dpr, mode);

	if (mode->parse.put_tree) {
		if (mode->echosent)	/* input sentence */
			fprintf(fp, "INPUT: %s\n", kpr->sent);
		put_tree(fp, kpr->root, 0, mode);
	}
}

/*---------- INTEGRATION GUIDE of HAM to your system -----------*/
/*                                                              */
/* 0. You should include 'header/ham.h' and 'header/parsek.h'.  */
/*                                                              */
/* 1. Required variables for parsing                            */
/*       char sentence[MAX];  ---> input sentence: KSC 5601     */
/*       HAM_RUNMODE mode;    ---> running mode of HAM          */
/*       KPAR_PSENT kpr;      ---> parsing result of HAM        */
/*                                                              */
/* 2. Call 'open_HAM_parse()' before the first call of parsing()*/
/*       It's argument is a diretory path of 'hangul.???'.      */
/*       It initializes global var.s and opens dic. files.      */
/*       Return value --- 0(normal), 1(dic.file open failure)   */
/*                                                              */
/* 3. Call 'parsing(sentence, &mode)'.                          */
/*       Morph & syntactic analysis of input sentence.          */
/*       Confirm parsing results by 'parsing_out()'.            */
/*                                                              */
/* 4. Call 'close_HAM()_parse', if 'parsing()' is ended.        */
/*                                                              */
/* Only 3 functions are required for integration to appl. system*/
/*        open_HAM_parse(), parsing(), close_HAM_parse()        */
/*                                                              */
/*--------------------------------------------------------------*/
main(argc, argv)
int argc; char *argv[];
{
	FILE *fpin=stdin, *fpout=stdout;

	char *optstr=NULL;  /* option string: e.g. "1aC2", "pVc" */
	HAM_RUNMODE mode;   /* HAM running mode: 'header/runmode.h' */

	unsigned char sent[SENTSIZE];	/* input sentence */
	KPAR_PSENT kpr;	/* parse tree: header/parsek.h */

	if (set_iofile_ptr(argc, argv, &fpin, &fpout))
		return 0;	/* I/O files open failed */
	if (argc > 1 && argv[1][0] == '-') optstr = argv[1]+1;

	/*=========================== Initialization ===========================*/
	if (open_HAM_parse(&mode, optstr, "hdic\\ham2000.ini")) {
		fprintf(stderr, "Error: dictionary-file path <%s>!", mode.dicpath);
		fprintf(stderr, " See <header/err-code.h> for code %d\n", mode.err_code);
		return 0;
	} else WELCOME;	/* welcome message: 'header/ham-par.h' */

	/* ɼ "ham2000.ini" Ǵ Ȯϴ  */
/*	if (set_options_from_file("/home/kang/ham2000.ini", &mode))
		fprintf(stderr, "WARNING: file not found --- <ham2000.ini>\n");
*/
	if (mode.autospac) load_bigram_data("hdic\\bigram.dic");	/* ڵ  */

	while (get_sent_or_line(fpin, sent, mode.inputmode)) {
		if (mode.autospac) adjust_word_spacing(sent, 1);	/* ڵ  */
		kpr = parsing(sent, &mode);
		parsing_out(fpout, kpr, &mode);	/* head-daughter parse tree */
	}

	/*=========================== Finalize HAM ===========================*/
	if (mode.autospac) free_bigram_data();	/* ڵ  */
	close_HAM_parse();	/* HAM is not used any more */

	if (fpin != stdin) fclose(fpin);
	if (fpout!= stdout)fclose(fpout);
	GOODBYE;	/* goodbye message: 'header/ham-par.h' */

	return 0;
}
/*--------------------------- end of parse.c ---------------------------*/
